@@ -95,9 +95,6 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
         i.next();
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
         switch (i.value().type()) {
-#else
-        switch (i.value().typeId()) {
-#endif
         case QVariant::Int:
             setProducerProperty(i.key(), i.value().toInt());
             break;
@@ -108,6 +105,19 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
             setProducerProperty(i.key(), i.value().toString());
             break;
         }
+#else
+        switch (i.value().typeId()) {
+        case QMetaType::Int:
+            setProducerProperty(i.key(), i.value().toInt());
+            break;
+        case QMetaType::Double:
+            setProducerProperty(i.key(), i.value().toDouble());
+            break;
+        default:
+            setProducerProperty(i.key(), i.value().toString());
+            break;
+        }
+#endif
     }
     m_tempProps.clear();
     int id = m_controllerBinId.toInt();
@@ -165,11 +175,12 @@ void ClipController::addMasterProducer(const std::shared_ptr<Mlt::Producer> &pro
     connectEffectStack();
 }
 
-const QString ClipController::producerXml(Mlt::Producer producer, bool includeMeta, bool includeProfile)
+const QByteArray ClipController::producerXml(Mlt::Producer producer, bool includeMeta, bool includeProfile)
 {
+    QReadLocker lock(&pCore->xmlMutex);
     Mlt::Consumer c(*producer.profile(), "xml", "string");
     if (!producer.is_valid()) {
-        return QString();
+        return QByteArray();
     }
     c.set("time_format", "frames");
     if (!includeMeta) {
@@ -183,14 +194,14 @@ const QString ClipController::producerXml(Mlt::Producer producer, bool includeMe
     c.set("root", "/");
     c.connect(producer);
     c.run();
-    return QString::fromUtf8(c.get("string"));
+    return QByteArray(c.get("string"));
 }
 
 void ClipController::getProducerXML(QDomDocument &document, bool includeMeta, bool includeProfile)
 {
     // TODO refac this is a probable duplicate with Clip::xml
     if (m_masterProducer) {
-        QString xml = producerXml(m_masterProducer->parent(), includeMeta, includeProfile);
+        const QByteArray xml = producerXml(m_masterProducer->parent(), includeMeta, includeProfile);
         document.setContent(xml);
     } else {
         if (!m_temporaryUrl.isEmpty()) {
@@ -1033,6 +1044,15 @@ bool ClipController::addEffect(const QString &effectId, stringMap params)
     return m_effectStack->appendEffect(effectId, true, params);
 }
 
+bool ClipController::copyEffectWithUndo(const std::shared_ptr<EffectStackModel> &stackModel, int rowId, Fun &undo, Fun &redo)
+{
+    return m_effectStack->copyEffectWithUndo(stackModel->getEffectStackRow(rowId),
+                                             !m_hasAudio   ? PlaylistState::VideoOnly
+                                             : !m_hasVideo ? PlaylistState::AudioOnly
+                                                           : PlaylistState::Disabled,
+                                             undo, redo);
+}
+
 bool ClipController::copyEffect(const std::shared_ptr<EffectStackModel> &stackModel, int rowId)
 {
     m_effectStack->copyEffect(stackModel->getEffectStackRow(rowId),
